@@ -63,9 +63,11 @@ def BaseNestedFieldSerializerFactory(
     def join_words(words, many='are', single='is'):
         word_list = ["`" + word + "`" for word in words]
         sentence = " & ".join([", ".join(word_list[:-1]), word_list[-1]])
-        if len(words) < 2:
+        if len(words) > 1:
+            return "%s %s" % (many, sentence)
+        elif len(words) == 1:
             return "%s %s" % (single, word_list[0])
-        return "%s %s" % (many, sentence)
+        return "%s %s" % (single, "[]")
 
     if not set(create_ops).issubset(set(CREATE_OPERATIONS)):
         msg = (
@@ -180,7 +182,7 @@ def BaseNestedFieldSerializerFactory(
                 except ValidationError as e:
                     detail = {operation: e.detail}
                     code = e.get_codes()
-                    raise ValidationError(detail, code) from None
+                    raise ValidationError(detail, code=code) from None
                 except KeyError:
                     msg = (
                         "`%s` is not a valid operation, valid operation(s) "
@@ -201,7 +203,7 @@ def BaseNestedFieldSerializerFactory(
                 except ValidationError as e:
                     detail = {operation: e.detail}
                     code = e.get_codes()
-                    raise ValidationError(detail, code) from None
+                    raise ValidationError(detail, code=code) from None
                 except KeyError:
                     msg = (
                         "`%s` is not a valid operation, valid operations(s) "
@@ -327,8 +329,8 @@ def NestedFieldWraper(*args, **kwargs):
     serializer_class = kwargs["serializer_class"]
 
     serializer_validation_kwargs = {**factory['kwargs']}
-    # TODO: Find all non validation related kwargs to remove(below are just few)
 
+    # TODO: Find all non validation related kwargs to remove(below are just few)
     # Remove non validation related kwargs from `valdation_kwargs`
     non_validation_kwargs = [
         'many', 'data', 'instance', 'context', 'fields',
