@@ -2,8 +2,6 @@ import re
 
 from pypeg2 import List, contiguous, csl, name, optional, parse
 
-from .exceptions import QueryFormatError
-
 
 class Alias(List):
     grammar = name(), ':'
@@ -125,12 +123,9 @@ class Block(List):
 ParentField.grammar = IncludedField, Block
 
 
-class Parser(object):
-    def __init__(self, query):
-        self._query = query
-
-    def get_parsed(self):
-        parse_tree = parse(self._query, Block)
+class QueryParser(object):
+    def parse(self, query):
+        parse_tree = parse(query, Block)
         return self._transform_block(parse_tree)
 
     def _transform_block(self, block):
@@ -165,36 +160,15 @@ class Parser(object):
                 # include all fields
                 fields["include"].append("*")
 
-        if fields["exclude"]:
-            # fields['include'] should contain only nested fields
-
-            # We should add `*` operator in fields['include']
-            add_include_all_operator = True
-            for field in fields["include"]:
-                if field == "*":
-                    # `*` operator is alredy in fields['include']
-                    add_include_all_operator = False
-                    continue
-
-                if isinstance(field, str):
-                    # Including and excluding fields on the same field level
-                    msg = (
-                        "Can not include and exclude fields on the same "
-                        "field level"
-                    )
-                    raise QueryFormatError(msg)
-
-            if add_include_all_operator:
-                # Make sure we include * operator
-                fields["include"].append("*")
+        if fields["exclude"] and "*" not in fields["include"]:
+            fields["include"].append("*")
         return fields
 
     def _transform_field(self, field):
         # A field may be a parent or included field or excluded field
         if isinstance(field, ParentField):
             return self._transform_parent_field(field)
-        elif isinstance(field, (IncludedField, ExcludedField, AllFields)):
-            return field
+        return field
 
     def _transform_parent_field(self, parent_field):
         parent_field_name = str(parent_field.alias_or_name)
